@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2009,2010,2011 Red Hat, Inc.
+ * Copyright (C) 2009,2010,2011,2014,2015,2017 Red Hat, Inc.
  * 
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,6 +20,7 @@
 #include <sys/types.h>
 #include <sys/select.h>
 #include <sys/socket.h>
+#include <sys/wait.h>
 #include <errno.h>
 #include <stdlib.h>
 #include <signal.h>
@@ -27,37 +28,54 @@
 #include <time.h>
 #include <unistd.h>
 
+#include <dbus/dbus.h>
+
 #include <talloc.h>
 #include <tevent.h>
 
-#include <dbus/dbus.h>
-
 #include "cm.h"
 #include "log.h"
 #include "iterate.h"
 #include "netlink.h"
 #include "store.h"
 #include "store-int.h"
+#include "subproc.h"
+#include "tdbus.h"
 #include "tdbush.h"
 #include "tm.h"
 
 struct cm_context {
-	int n_entries, should_quit;
+	int should_quit;
+	int n_entries;
 	struct cm_store_entry **entries;
+	int n_cas;
+	struct cm_store_ca **cas;
 	struct cm_event {
 		void *iterate_state;
 		void *next_event;
-	} *events;
-	int n_cas;
-	struct cm_store_ca **cas;
+	} *entry_events;
+	struct cm_ca_event {
+		void *iterate_state[cm_ca_phase_invalid];
+		void *next_event[cm_ca_phase_invalid];
+	} *ca_events;
 	int netlink;
 	void *netlink_tfd, *netlink_delayed_event;
 	int idle_timeout;
 	void *idle_event, *conn_ptr;
+	char *server_address;
+	struct {
+		void *tfd;
+		char *command;
+		int fd;
+		struct cm_subproc_state *state;
+	} gate;
 };
 
-static void *cm_service_one(struct cm_context *context,
-			    struct timeval *now, int i);
+static void *cm_service_entry(struct cm_context *context,
+			      struct timeval *now, int i);
+static void *cm_service_ca(struct cm_context *context,
+			   struct timeval *now, int i,
+			   enum cm_ca_phase phase);
 static void cm_fd_h(struct tevent_context *ec, struct tevent_fd *fde,
 		    uint16_t flags, void *pvt);
 static void cm_timer_h(struct tevent_context *ec, struct tevent_timer *te,
@@ -67,14 +85,16 @@ static void cm_break_h(struct tevent_context *ec, struct tevent_signal *se,
 static void cm_netlink_fd_h(struct tevent_context *ec, struct tevent_fd *fde,
 			    uint16_t flags, void *pvt);
 static void cm_timeout_h(struct tevent_context *ec, struct tevent_timer *te,
-		         struct timeval current_time, void *pvt);
+			 struct timeval current_time, void *pvt);
 
 int
 cm_init(struct tevent_context *parent, struct cm_context **context,
-	int idle_timeout)
+	int idle_timeout, const char *gate_command)
 {
 	struct cm_context *ctx;
 	int i, j;
+	enum cm_ca_phase phase;
+
 	*context = NULL;
 	ctx = talloc_ptrtype(parent, ctx);
 	if (ctx == NULL) {
@@ -88,18 +108,27 @@ cm_init(struct tevent_context *parent, struct cm_context **context,
 	}
 	ctx->n_entries = i;
 	/* Allocate space for the tevents for each entry. */
-	ctx->events = talloc_array_ptrtype(ctx, ctx->events, ctx->n_entries);
-	if (ctx->events == NULL) {
+	ctx->entry_events = talloc_array_ptrtype(ctx, ctx->entry_events,
+						 ctx->n_entries);
+	if (ctx->entry_events == NULL) {
 		talloc_free(ctx);
 		return ENOMEM;
 	}
-	memset(ctx->events, 0, sizeof(ctx->events[0]) * ctx->n_entries);
+	memset(ctx->entry_events, 0,
+	       sizeof(ctx->entry_events[0]) * ctx->n_entries);
 	/* Read the list of known CAs. */
 	ctx->cas = cm_store_get_all_cas(ctx);
 	for (i = 0; (ctx->cas != NULL) && (ctx->cas[i] != NULL); i++) {
 		continue;
 	}
 	ctx->n_cas = i;
+	/* Allocate space for the tevents for each CA. */
+	ctx->ca_events = talloc_array_ptrtype(ctx, ctx->ca_events, ctx->n_cas);
+	if (ctx->ca_events == NULL) {
+		talloc_free(ctx);
+		return ENOMEM;
+	}
+	memset(ctx->ca_events, 0, sizeof(ctx->ca_events[0]) * ctx->n_cas);
 	/* Handle things which should get us to quit. */
 	tevent_add_signal(parent, ctx, SIGHUP, 0, cm_break_h, ctx);
 	tevent_add_signal(parent, ctx, SIGINT, 0, cm_break_h, ctx);
@@ -107,20 +136,49 @@ cm_init(struct tevent_context *parent, struct cm_context **context,
 	/* Be ready for an idle timeout. */
 	ctx->idle_timeout = idle_timeout;
 	ctx->idle_event = NULL;
+	/* Be ready to launch a gating command. */
+	if (gate_command != NULL) {
+		ctx->gate.command = talloc_strdup(ctx, gate_command);
+	}
 	/* Initialize state tracking, but don't set things in motion yet. */
 	for (i = 0; i < ctx->n_entries; i++) {
-		memset(&ctx->events[i], 0, sizeof(ctx->events[i]));
-		if (cm_iterate_init(ctx->entries[i],
-				    &ctx->events[i].iterate_state) != 0) {
+		memset(&ctx->entry_events[i], 0, sizeof(ctx->entry_events[i]));
+		if (cm_iterate_entry_init(ctx->entries[i],
+					  &ctx->entry_events[i].iterate_state) != 0) {
 			for (j = 0; j < i; j++) {
-				cm_iterate_done(ctx->entries[j],
-						ctx->events[j].iterate_state);
-				ctx->events[j].iterate_state = NULL;
+				cm_iterate_entry_done(ctx->entries[j],
+						      ctx->entry_events[j].iterate_state);
+				ctx->entry_events[j].iterate_state = NULL;
 			}
 			talloc_free(ctx);
 			return ENOMEM;
 		}
 	}
+	for (i = 0; i < ctx->n_cas; i++) {
+		memset(&ctx->ca_events[i], 0, sizeof(ctx->ca_events[i]));
+		for (phase = 0; phase < cm_ca_phase_invalid; phase++) {
+			if (cm_iterate_ca_init(ctx->cas[i], phase,
+					       &ctx->ca_events[i].iterate_state[phase]) != 0) {
+				do {
+					phase--;
+					cm_iterate_ca_done(ctx->cas[i],
+							   ctx->ca_events[i].iterate_state[phase]);
+					ctx->ca_events[i].iterate_state[phase] = NULL;
+				} while (phase > 0);
+				for (j = 0; j < i; j++) {
+					phase = cm_ca_phase_invalid;
+					do {
+						phase--;
+						cm_iterate_ca_done(ctx->cas[j],
+								   ctx->ca_events[j].iterate_state[phase]);
+						ctx->ca_events[j].iterate_state[phase] = NULL;
+					} while (phase > 0);
+				}
+				talloc_free(ctx);
+				return ENOMEM;
+			}
+		}
+	}
 	/* Start draining the netlink socket so that it doesn't get backed up
 	 * waiting for us to read notifications. */
 	ctx->netlink = cm_netlink_socket();
@@ -140,16 +198,31 @@ cm_timer_h(struct tevent_context *ec, struct tevent_timer *te,
 	   struct timeval current_time, void *pvt)
 {
 	struct cm_context *context = pvt;
-	int i;
+	int i, j;
+	enum cm_ca_phase phase;
+
 	for (i = 0; i < context->n_entries; i++) {
-		if (context->events[i].next_event == te) {
+		if (context->entry_events[i].next_event == te) {
 			talloc_free(te);
-			context->events[i].next_event = cm_service_one(context,
-								       NULL, i);
+			context->entry_events[i].next_event =
+				cm_service_entry(context, NULL, i);
 			break;
 		}
 	}
-	if (i >= context->n_entries) {
+	for (j = 0; j < context->n_cas; j++) {
+		for (phase = 0; phase < cm_ca_phase_invalid; phase++) {
+			if (context->ca_events[j].next_event[phase] == te) {
+				talloc_free(te);
+				context->ca_events[j].next_event[phase] =
+					cm_service_ca(context, NULL, j, phase);
+				break;
+			}
+		}
+		if (phase < cm_ca_phase_invalid) {
+			break;
+		}
+	}
+	if ((i >= context->n_entries) && (j >= context->n_cas)) {
 		cm_log(3, "Bug: unowned timer fired.\n");
 	}
 }
@@ -159,6 +232,7 @@ cm_timeout_h(struct tevent_context *ec, struct tevent_timer *te,
 	     struct timeval current_time, void *pvt)
 {
 	struct cm_context *context = pvt;
+
 	if (context->idle_event != NULL) {
 		talloc_free(context->idle_event);
 		context->idle_event = NULL;
@@ -173,6 +247,7 @@ void
 cm_reset_timeout(struct cm_context *context)
 {
 	struct timeval now, then;
+
 	if (context->idle_event != NULL) {
 		cm_log(3, "Clearing previously-set idle timer.\n");
 		talloc_free(context->idle_event);
@@ -189,6 +264,10 @@ cm_reset_timeout(struct cm_context *context)
 						       then,
 						       cm_timeout_h,
 						       context);
+	} else if (context->idle_timeout > 0) {
+		cm_log(1, "There are active certificates and requests, "
+		       "ignoring idle timeout.\n");
+		context->idle_timeout = 0;
 	}
 }
 
@@ -197,16 +276,31 @@ cm_fd_h(struct tevent_context *ec,
 	struct tevent_fd *fde, uint16_t flags, void *pvt)
 {
 	struct cm_context *context = pvt;
-	int i;
+	int i, j;
+	enum cm_ca_phase phase;
+
 	for (i = 0; i < context->n_entries; i++) {
-		if (context->events[i].next_event == fde) {
+		if (context->entry_events[i].next_event == fde) {
 			talloc_free(fde);
-			context->events[i].next_event = cm_service_one(context,
-								       NULL, i);
+			context->entry_events[i].next_event =
+				cm_service_entry(context, NULL, i);
 			break;
 		}
 	}
-	if (i >= context->n_entries) {
+	for (j = 0; j < context->n_cas; j++) {
+		for (phase = 0; phase < cm_ca_phase_invalid; phase++) {
+			if (context->ca_events[j].next_event[phase] == fde) {
+				talloc_free(fde);
+				context->ca_events[j].next_event[phase] =
+					cm_service_ca(context, NULL, j, phase);
+				break;
+			}
+		}
+		if (phase < cm_ca_phase_invalid) {
+			break;
+		}
+	}
+	if ((i >= context->n_entries) && (j >= context->n_cas)) {
 		cm_log(3, "Bug: unowned FD watch fired.\n");
 	}
 }
@@ -226,18 +320,35 @@ cm_netlink_delayed_h(struct tevent_context *ec, struct tevent_timer *te,
 {
 	struct cm_context *ctx = pvt;
 	int i;
+	enum cm_ca_phase phase;
+
 	for (i = 0; i < ctx->n_entries; i++) {
-		if (ctx->events[i].next_event != NULL) {
+		if (ctx->entry_events[i].next_event != NULL) {
 			switch (ctx->entries[i]->cm_state) {
 			case CM_CA_UNREACHABLE:
-				cm_restart_one(ctx,
-					       ctx->entries[i]->cm_nickname);
+				cm_restart_entry(ctx,
+						 ctx->entries[i]->cm_nickname);
 				break;
 			default:
 				break;
 			}
 		}
 	}
+	for (i = 0; i < ctx->n_cas; i++) {
+		for (phase = 0; phase < cm_ca_phase_invalid; phase++) {
+			if (ctx->ca_events[i].iterate_state[phase] != NULL) {
+				switch (ctx->cas[i]->cm_ca_state[phase]) {
+				case CM_CA_DATA_UNREACHABLE:
+					cm_restart_ca(ctx,
+						      ctx->cas[i]->cm_nickname,
+						      phase);
+					break;
+				default:
+					break;
+				}
+			}
+		}
+	}
 	if (te == ctx->netlink_delayed_event) {
 		talloc_free(ctx->netlink_delayed_event);
 		ctx->netlink_delayed_event = NULL;
@@ -307,7 +418,7 @@ cm_find_ca_by_entry(struct cm_context *c, struct cm_store_entry *entry)
 }
 
 static void *
-cm_service_one(struct cm_context *context, struct timeval *current_time, int i)
+cm_service_entry(struct cm_context *context, struct timeval *current_time, int i)
 {
 	int ret, delay, fd;
 	struct timeval now, then;
@@ -320,15 +431,17 @@ cm_service_one(struct cm_context *context, struct timeval *current_time, int i)
 		now = tevent_timeval_current();
 	}
 	fd = -1;
-	ret = cm_iterate(context->entries[i],
-			 cm_find_ca_by_entry(context, context->entries[i]),
-			 context,
-			 &cm_get_ca_by_index,
-			 &cm_get_n_cas,
-			 &cm_tdbush_property_emit_entry_saved_cert,
-			 &cm_tdbush_property_emit_entry_changes,
-			 context->events[i].iterate_state,
-			 &when, &delay, &fd);
+	ret = cm_iterate_entry(context->entries[i],
+			       cm_find_ca_by_entry(context, context->entries[i]),
+			       context,
+			       &cm_get_ca_by_index,
+			       &cm_get_n_cas,
+			       &cm_get_entry_by_index,
+			       &cm_get_n_entries,
+			       &cm_tdbush_property_emit_entry_saved_cert,
+			       &cm_tdbush_property_emit_entry_changes,
+			       context->entry_events[i].iterate_state,
+			       &when, &delay, &fd);
 	t = NULL;
 	if (ret == 0) {
 		switch (when) {
@@ -386,6 +499,95 @@ cm_service_one(struct cm_context *context, struct timeval *current_time, int i)
 	return t;
 }
 
+static void *
+cm_service_ca(struct cm_context *context, struct timeval *current_time, int i,
+	      enum cm_ca_phase phase)
+{
+	int ret, delay, fd;
+	struct timeval now, then;
+	enum cm_time when;
+	void *t;
+
+	if (current_time != NULL) {
+		now = *current_time;
+	} else {
+		now = tevent_timeval_current();
+	}
+	fd = -1;
+	ret = cm_iterate_ca(context->cas[i],
+			    context,
+			    &cm_get_ca_by_index,
+			    &cm_get_n_cas,
+			    &cm_get_entry_by_index,
+			    &cm_get_n_entries,
+			    &cm_tdbush_property_emit_ca_changes,
+			    context->ca_events[i].iterate_state[phase],
+			    &when, &delay, &fd);
+	t = NULL;
+	if (ret == 0) {
+		switch (when) {
+		case cm_time_now:
+			t = tevent_add_timer(talloc_parent(context), context,
+					     now, cm_timer_h, context);
+			cm_log(3, "Will revisit %s('%s').%s now.\n",
+			       context->cas[i]->cm_busname,
+			       context->cas[i]->cm_nickname,
+			       cm_store_ca_phase_as_string(phase));
+			break;
+		case cm_time_soon:
+			then = tevent_timeval_add(&now, CM_DELAY_SOON, 0);
+			t = tevent_add_timer(talloc_parent(context), context,
+					     then, cm_timer_h, context);
+			cm_log(3, "Will revisit %s('%s').%s soon.\n",
+			       context->cas[i]->cm_busname,
+			       context->cas[i]->cm_nickname,
+			       cm_store_ca_phase_as_string(phase));
+			break;
+		case cm_time_soonish:
+			then = tevent_timeval_add(&now, CM_DELAY_SOONISH, 0);
+			t = tevent_add_timer(talloc_parent(context), context,
+					     then, cm_timer_h, context);
+			cm_log(3, "Will revisit %s('%s').%s soonish.\n",
+			       context->cas[i]->cm_busname,
+			       context->cas[i]->cm_nickname,
+			       cm_store_ca_phase_as_string(phase));
+			break;
+		case cm_time_delay:
+			then = tevent_timeval_add(&now, delay, 0);
+			t = tevent_add_timer(talloc_parent(context), context,
+					     then, cm_timer_h, context);
+			cm_log(3, "Will revisit %s('%s').%s in %d seconds.\n",
+			       context->cas[i]->cm_busname,
+			       context->cas[i]->cm_nickname,
+			       cm_store_ca_phase_as_string(phase),
+			       delay);
+			break;
+		case cm_time_no_time:
+			if (fd != -1) {
+				t = tevent_add_fd(talloc_parent(context),
+						  context,
+						  fd, TEVENT_FD_READ,
+						  cm_fd_h, context);
+				cm_log(3, "Will revisit %s('%s').%s on "
+				       "traffic from %d.\n",
+				       context->cas[i]->cm_busname,
+				       context->cas[i]->cm_nickname,
+				       cm_store_ca_phase_as_string(phase),
+				       fd);
+			} else {
+				cm_log(3, "Waiting for instructions for "
+				       "%s('%s').%s.\n",
+				       context->cas[i]->cm_busname,
+				       context->cas[i]->cm_nickname,
+				       cm_store_ca_phase_as_string(phase));
+				t = NULL;
+			}
+			break;
+		}
+	}
+	return t;
+}
+
 int
 cm_keep_going(struct cm_context *context)
 {
@@ -400,6 +602,7 @@ cm_add_entry(struct cm_context *context, struct cm_store_entry *new_entry)
 	int i;
 	time_t now;
 	char timestamp[15];
+
 	/* Check for duplicates and count the number of entries we're already
 	 * managing. */
 	if (new_entry->cm_nickname != NULL) {
@@ -428,39 +631,31 @@ cm_add_entry(struct cm_context *context, struct cm_store_entry *new_entry)
 		new_entry->cm_nickname = talloc_strdup(new_entry,
 						       new_entry->cm_nickname);
 	}
-	/* Allocate storage for a new entry array. */
+	/* Resize the entry array. */
 	events = NULL;
-	entries = talloc_array(context, struct cm_store_entry *,
-			       context->n_entries + 1);
+	entries = talloc_realloc(context, context->entries,
+				 struct cm_store_entry *,
+				 context->n_entries + 1);
 	if (entries != NULL) {
-		/* Allocate storage for a new entry state array. */
-		events = talloc_array(context, struct cm_event,
-				      context->n_entries + 1);
+		/* Resize the entry state array. */
+		events = talloc_realloc(context, context->entry_events,
+					struct cm_event,
+					context->n_entries + 1);
 		if (events != NULL) {
-			/* Copy the entries to the new arrays. */
-			for (i = 0; i < context->n_entries; i++) {
-				talloc_steal(entries, context->entries[i]);
-				entries[i] = context->entries[i];
-			}
-			/* The pointers in this structure belong to the tevent
-			 * context, so we don't need to worry about reparenting
-			 * them. */
-			memcpy(events, context->events,
-			       sizeof(context->events[0]) * context->n_entries);
-			/* Add the new members. */
+			/* Add the new entry to the array. */
 			talloc_steal(entries, new_entry);
 			entries[context->n_entries] = new_entry;
+			/* Clear the new entry event. */
 			memset(&events[context->n_entries], 0,
 			       sizeof(events[context->n_entries]));
-			/* Reset the pointers. */
-			talloc_free(context->entries);
+			/* Update the pointers. */
 			context->entries = entries;
-			talloc_free(context->events);
-			context->events = events;
-			/* Reset the recorded count of entries. */
+			context->entry_events = events;
+			/* Update the recorded count of entries. */
 			context->n_entries++;
 		} else {
-			talloc_free(entries);
+			/* At least don't sabotage things. */
+			context->entries = entries;
 			entries = NULL;
 		}
 	}
@@ -468,8 +663,8 @@ cm_add_entry(struct cm_context *context, struct cm_store_entry *new_entry)
 	if ((entries != NULL) && (events != NULL)) {
 		/* Prepare to set this entry in motion. */
 		i = context->n_entries - 1;
-		if (cm_start_one(context,
-				 context->entries[i]->cm_nickname) == FALSE) {
+		if (cm_start_entry(context,
+				   context->entries[i]->cm_nickname) == FALSE) {
 			cm_log(3, "Error starting %s('%s'), please retry.\n",
 			       context->entries[i]->cm_busname,
 			       context->entries[i]->cm_nickname);
@@ -497,6 +692,7 @@ static int
 cm_find_ca_by_nickname(struct cm_context *context, const char *nickname)
 {
 	int i;
+
 	for (i = 0; i < context->n_cas; i++) {
 		if (strcmp(context->cas[i]->cm_nickname, nickname) == 0) {
 			return i;
@@ -505,21 +701,126 @@ cm_find_ca_by_nickname(struct cm_context *context, const char *nickname)
 	return -1;
 }
 
+static void
+cm_gate_fd_h(struct tevent_context *ec, struct tevent_fd *fde,
+	     uint16_t flags, void *pvt)
+{
+	struct cm_context *ctx = pvt;
+	int length, status;
+	const char *msg;
+
+	talloc_free(ctx->gate.tfd);
+	if (cm_subproc_ready(ctx->gate.state) == 0) {
+		msg = cm_subproc_get_msg(ctx->gate.state, &length);
+		if (length > 0) {
+			cm_log(0, "Failed to start command '%s': %s.\n",
+			       ctx->gate.command,
+			       strerror((unsigned int) msg[0]));
+		} else {
+			status = cm_subproc_get_exitstatus(ctx->gate.state);
+			if (WIFEXITED(status)) {
+				cm_log(1, "Command '%s' exited, status %d.\n",
+				       ctx->gate.command, WEXITSTATUS(status));
+			} else {
+				cm_log(0, "Command '%s' exited abnormally.\n",
+				       ctx->gate.command);
+			}
+		}
+		ctx->should_quit++;
+		ctx->gate.tfd = NULL;
+	} else {
+		cm_log(1, "Command '%s' output error data, but is still "
+		       "running.\n", ctx->gate.command);
+		ctx->gate.tfd = tevent_add_fd(ec, ctx, ctx->gate.fd,
+					      TEVENT_FD_READ, cm_gate_fd_h,
+					      ctx);
+	}
+}
+
+static int
+cm_gate_run(int fd, struct cm_store_ca *ca, struct cm_store_entry *e,
+	    void *data)
+{
+	struct cm_context *ctx = data;
+	char **argv;
+	const char *error = NULL;
+	unsigned char u;
+
+	cm_subproc_mark_most_cloexec(fd, STDOUT_FILENO, STDERR_FILENO);
+	argv = cm_subproc_parse_args(NULL, ctx->gate.command, &error);
+	if (argv == NULL) {
+		cm_log(1, "Error parsing '%s'.\n", ctx->gate.command);
+		return -1;
+	}
+	cm_log(1, "Running gate command \"%s\" (\"%s\").\n", argv[0],
+	       ctx->gate.command);
+	if (ctx->server_address != NULL) {
+		setenv(CERTMONGER_PVT_ADDRESS_ENV, ctx->server_address, 1);
+	}
+	signal(SIGHUP, SIG_DFL);
+	signal(SIGINT, SIG_DFL);
+	signal(SIGTERM, SIG_DFL);
+	execvp(argv[0], argv);
+	u = errno;
+	if (write(fd, &u, 1) != 1) {
+		cm_log(1, "Error sending exec() error to parent.\n");
+	}
+	return u;
+}
+
 int
 cm_start_all(struct cm_context *context)
 {
 	int i;
+	enum cm_ca_phase phase;
+
+	if (context->gate.command != NULL) {
+		context->gate.state = cm_subproc_start(cm_gate_run, context,
+						       NULL, NULL, context);
+		if (context->gate.state == NULL) {
+			cm_log(1, "Error starting '%s', please try again.\n",
+			       context->gate.command);
+			return -1;
+		}
+		i = cm_subproc_get_fd(context->gate.state);
+		if (i == -1) {
+			cm_log(1, "Error starting '%s', please try again.\n",
+			       context->gate.command);
+			return -1;
+		}
+		context->gate.fd = i;
+		context->gate.tfd = tevent_add_fd(talloc_parent(context),
+						  context, i, TEVENT_FD_READ,
+						  cm_gate_fd_h, context);
+		cm_log(3, "Command '%s' on FD %d.\n", context->gate.command, i);
+	}
 	for (i = 0; i < context->n_entries; i++) {
-		if ((context->events[i].iterate_state == NULL) &&
-		    (cm_iterate_init(context->entries[i],
-				     &context->events[i].iterate_state)) != 0) {
+		if ((context->entry_events[i].iterate_state == NULL) &&
+		    (cm_iterate_entry_init(context->entries[i],
+					   &context->entry_events[i].iterate_state)) != 0) {
 			cm_log(1, "Error starting %s('%s'), "
 			       "please try again.\n",
 			       context->entries[i]->cm_busname,
 			       context->entries[i]->cm_nickname);
 		} else {
-			context->events[i].next_event = cm_service_one(context,
-								       NULL, i);
+			context->entry_events[i].next_event =
+				cm_service_entry(context, NULL, i);
+		}
+	}
+	for (i = 0; i < context->n_cas; i++) {
+		for (phase = 0; phase < cm_ca_phase_invalid; phase++) {
+			if ((context->ca_events[i].iterate_state[phase] == NULL) &&
+			    (cm_iterate_ca_init(context->cas[i], phase,
+						&context->ca_events[i].iterate_state[phase])) != 0) {
+				cm_log(1, "Error starting %s('%s')-%s, "
+				       "please try again.\n",
+				       context->cas[i]->cm_busname,
+				       context->cas[i]->cm_nickname,
+				       cm_store_ca_phase_as_string(phase));
+			} else {
+				context->ca_events[i].next_event[phase] =
+					cm_service_ca(context, NULL, i, phase);
+			}
 		}
 	}
 	cm_reset_timeout(context);
@@ -530,29 +831,42 @@ void
 cm_stop_all(struct cm_context *context)
 {
 	int i;
+	enum cm_ca_phase phase;
+
 	for (i = 0; i < context->n_entries; i++) {
-		talloc_free(context->events[i].next_event);
-		context->events[i].next_event = NULL;
-		cm_iterate_done(context->entries[i],
-				context->events[i].iterate_state);
-		context->events[i].iterate_state = NULL;
+		talloc_free(context->entry_events[i].next_event);
+		context->entry_events[i].next_event = NULL;
+		cm_iterate_entry_done(context->entries[i],
+				      context->entry_events[i].iterate_state);
+		context->entry_events[i].iterate_state = NULL;
 		cm_store_entry_save(context->entries[i]);
 	}
 	for (i = 0; i < context->n_cas; i++) {
+		for (phase = 0; phase < cm_ca_phase_invalid; phase++) {
+			talloc_free(context->ca_events[i].next_event[phase]);
+			context->ca_events[i].next_event[phase] = NULL;
+			cm_iterate_ca_done(context->cas[i],
+					   context->ca_events[i].iterate_state[phase]);
+			context->ca_events[i].iterate_state[phase] = NULL;
+		}
 		cm_store_ca_save(context->cas[i]);
 	}
+	if (context->gate.state != NULL) {
+		cm_subproc_done(context->gate.state);
+	}
 }
 
 dbus_bool_t
-cm_start_one(struct cm_context *context, const char *nickname)
+cm_start_entry(struct cm_context *context, const char *nickname)
 {
 	int i;
+
 	i = cm_find_entry_by_nickname(context, nickname);
 	if (i != -1) {
-		if (cm_iterate_init(context->entries[i],
-				    &context->events[i].iterate_state) == 0) {
-			context->events[i].next_event = cm_service_one(context,
-								       NULL, i);
+		if (cm_iterate_entry_init(context->entries[i],
+					  &context->entry_events[i].iterate_state) == 0) {
+			context->entry_events[i].next_event =
+				cm_service_entry(context, NULL, i);
 			cm_log(3, "Started %s('%s').\n",
 			       context->entries[i]->cm_busname, nickname);
 			return TRUE;
@@ -568,16 +882,17 @@ cm_start_one(struct cm_context *context, const char *nickname)
 }
 
 dbus_bool_t
-cm_stop_one(struct cm_context *context, const char *nickname)
+cm_stop_entry(struct cm_context *context, const char *nickname)
 {
 	int i;
+
 	i = cm_find_entry_by_nickname(context, nickname);
 	if (i != -1) {
-		talloc_free(context->events[i].next_event);
-		context->events[i].next_event = NULL;
-		cm_iterate_done(context->entries[i],
-				context->events[i].iterate_state);
-		context->events[i].iterate_state = NULL;
+		talloc_free(context->entry_events[i].next_event);
+		context->entry_events[i].next_event = NULL;
+		cm_iterate_entry_done(context->entries[i],
+				      context->entry_events[i].iterate_state);
+		context->entry_events[i].iterate_state = NULL;
 		cm_store_entry_save(context->entries[i]);
 		cm_log(3, "Stopped %s('%s').\n",
 		       context->entries[i]->cm_busname, nickname);
@@ -592,7 +907,7 @@ int
 cm_remove_entry(struct cm_context *context, const char *nickname)
 {
 	int i, rv = -1;
-	if (cm_stop_one(context, nickname)) {
+	if (cm_stop_entry(context, nickname)) {
 		i = cm_find_entry_by_nickname(context, nickname);
 		if (i != -1) {
 			if (cm_store_entry_delete(context->entries[i]) == 0) {
@@ -604,10 +919,10 @@ cm_remove_entry(struct cm_context *context, const char *nickname)
 					context->entries + i + 1,
 					(context->n_entries - i - 1) *
 					sizeof(context->entries[i]));
-				memmove(context->events + i,
-					context->events + i + 1,
+				memmove(context->entry_events + i,
+					context->entry_events + i + 1,
 					(context->n_entries - i - 1) *
-					sizeof(context->events[i]));
+					sizeof(context->entry_events[i]));
 				context->n_entries--;
 				rv = 0;
 			} else {
@@ -620,10 +935,28 @@ cm_remove_entry(struct cm_context *context, const char *nickname)
 }
 
 dbus_bool_t
-cm_restart_one(struct cm_context *context, const char *nickname)
+cm_restart_entry(struct cm_context *context, const char *nickname)
 {
-	return cm_stop_one(context, nickname) &&
-	       cm_start_one(context, nickname);
+	return cm_stop_entry(context, nickname) &&
+	       cm_start_entry(context, nickname);
+}
+
+dbus_bool_t
+cm_restart_entries_by_ca(struct cm_context *context, const char *nickname)
+{
+	struct cm_store_entry *entry;
+	dbus_bool_t status = FALSE, this;
+	int i, n = 0;
+
+	for (i = 0; i < context->n_entries; i++) {
+		entry = context->entries[i];
+		if ((entry->cm_ca_nickname != NULL) &&
+		    (strcmp(entry->cm_ca_nickname, nickname) == 0)) {
+			this = cm_restart_entry(context, entry->cm_nickname);
+			status = n++ ? this && status : this;
+		}
+	}
+	return status;
 }
 
 struct cm_store_entry *
@@ -669,9 +1002,12 @@ int
 cm_add_ca(struct cm_context *context, struct cm_store_ca *new_ca)
 {
 	struct cm_store_ca **cas;
+	struct cm_ca_event *events;
 	int i;
 	time_t now;
 	char timestamp[15];
+	enum cm_ca_phase phase;
+
 	/* Check for duplicates and count the number of CAs we're already
 	 * managing. */
 	if (new_ca->cm_nickname != NULL) {
@@ -701,26 +1037,96 @@ cm_add_ca(struct cm_context *context, struct cm_store_ca *new_ca)
 						    new_ca->cm_nickname);
 	}
 	/* Allocate storage for a new CA array. */
-	cas = talloc_array(context, struct cm_store_ca *, context->n_cas + 2);
-	if (cas != NULL) {
-		/* Copy the entries to the new arrays. */
-		for (i = 0; i < context->n_cas; i++) {
-			talloc_steal(cas, context->cas[i]);
-			cas[i] = context->cas[i];
-		}
+	cas = talloc_realloc(context, context->cas, struct cm_store_ca *,
+			     context->n_cas + 1);
+	events = talloc_realloc(context, context->ca_events,
+				struct cm_ca_event, context->n_cas + 1);
+	if ((cas != NULL) && (events != NULL)) {
 		/* Save this entry to the store. */
 		cm_store_ca_save(new_ca);
+		cas[context->n_cas] = new_ca;
 		talloc_steal(cas, new_ca);
-		cas[i++] = new_ca;
-		cas[i++] = NULL;
 		context->cas = cas;
-		/* Reset the recorded count of CAs. */
+		memset(&events[context->n_cas], 0,
+		       sizeof(events[context->n_cas]));
+		context->ca_events = events;
+		/* Update the recorded count of CAs. */
 		context->n_cas++;
+		/* Start the CA's data fetchers. */
+		for (phase = 0; phase < cm_ca_phase_invalid; phase++) {
+			cm_start_ca(context, new_ca->cm_nickname, phase);
+		}
 		return 0;
 	}
+	if (cas != NULL) {
+		context->cas = cas;
+	}
+	if (events != NULL) {
+		context->ca_events = events;
+	}
 	return -1;
 }
 
+dbus_bool_t
+cm_start_ca(struct cm_context *context, const char *nickname,
+	    enum cm_ca_phase phase)
+{
+	int i;
+
+	i = cm_find_ca_by_nickname(context, nickname);
+	if (i != -1) {
+		if (cm_iterate_ca_init(context->cas[i], phase,
+				       &context->ca_events[i].iterate_state[phase]) == 0) {
+			context->ca_events[i].next_event[phase] =
+				cm_service_ca(context, NULL, i, phase);
+			cm_log(3, "Started CA %s('%s')-%s.\n",
+			       context->cas[i]->cm_busname, nickname,
+			       cm_store_ca_phase_as_string(phase));
+			return TRUE;
+		} else {
+			cm_log(3, "Error starting CA %s('%s')-%s, please retry.\n",
+			       context->cas[i]->cm_busname, nickname,
+			       cm_store_ca_phase_as_string(phase));
+			return FALSE;
+		}
+	} else {
+		cm_log(3, "No CA matching nickname '%s'.\n", nickname);
+		return FALSE;
+	}
+}
+
+dbus_bool_t
+cm_stop_ca(struct cm_context *context, const char *nickname,
+	   enum cm_ca_phase phase)
+{
+	int i;
+
+	i = cm_find_ca_by_nickname(context, nickname);
+	if (i != -1) {
+		talloc_free(context->ca_events[i].next_event[phase]);
+		context->ca_events[i].next_event[phase] = NULL;
+		cm_iterate_ca_done(context->cas[i],
+				   context->ca_events[i].iterate_state[phase]);
+		context->ca_events[i].iterate_state[phase] = NULL;
+		cm_store_ca_save(context->cas[i]);
+		cm_log(3, "Stopped CA %s('%s')-%s.\n",
+		       context->cas[i]->cm_busname, nickname,
+		       cm_store_ca_phase_as_string(phase));
+		return TRUE;
+	} else {
+		cm_log(3, "No CA matching nickname '%s'.\n", nickname);
+		return FALSE;
+	}
+}
+
+dbus_bool_t
+cm_restart_ca(struct cm_context *context, const char *nickname,
+	      enum cm_ca_phase phase)
+{
+	return cm_stop_ca(context, nickname, phase) &&
+	       cm_start_ca(context, nickname, phase);
+}
+
 struct cm_store_ca *
 cm_get_ca_by_busname(struct cm_context *context, const char *name)
 {
@@ -764,17 +1170,32 @@ int
 cm_remove_ca(struct cm_context *context, const char *nickname)
 {
 	int i;
+	enum cm_ca_phase phase;
 	i = cm_find_ca_by_nickname(context, nickname);
 	if (i != -1) {
+		for (phase = 0; phase < cm_ca_phase_invalid; phase++) {
+			if (!cm_stop_ca(context, nickname, phase)) {
+				break;
+			}
+		}
+		if (phase != cm_ca_phase_invalid) {
+			cm_log(3, "Error stopping CA '%s'-%s, please retry.\n",
+			       nickname, cm_store_ca_phase_as_string(phase));
+			return -1;
+		}
 		if (cm_store_ca_delete(context->cas[i]) == 0) {
 			/* Free the entry. */
 			talloc_free(context->cas[i]);
-			/* Shorten up the arrays of entries and event
+			/* Shorten up the arrays of CAs and event
 			 * information. */
 			memmove(context->cas + i,
 				context->cas + i + 1,
 				(context->n_cas - i - 1) *
 				sizeof(context->cas[i]));
+			memmove(context->ca_events + i,
+				context->ca_events + i + 1,
+				(context->n_cas - i - 1) *
+				sizeof(context->ca_events[i]));
 			context->n_cas--;
 			return 0;
 		} else {
@@ -795,3 +1216,15 @@ cm_set_conn_ptr(struct cm_context *context, void *ptr)
 {
 	context->conn_ptr = ptr;
 }
+
+const char *
+cm_get_server_address(struct cm_context *context)
+{
+	return context->server_address;
+}
+
+void
+cm_set_server_address(struct cm_context *context, const char *address)
+{
+	context->server_address = talloc_strdup(context, address);
+}
