@@ -33,6 +33,8 @@
 #include <nss.h>
 #include <cert.h>
 
+#include <jansson.h>
+
 #include <dbus/dbus.h>
 
 #include <talloc.h>
@@ -49,6 +51,7 @@
 #include "util.h"
 #include "util-m.h"
 #include "util-n.h"
+#include "util-ipa.h"
 
 #ifdef ENABLE_NLS
 #include <libintl.h>
@@ -61,6 +64,15 @@
 #include "dogtag-ipa.h"
 #endif
 
+enum op_type {
+	op_none, op_submit, op_check, op_approve, op_retrieve, op_profiles
+};
+
+struct options {
+	char *name;
+	char *value;
+};
+
 static char *
 statevar(const char *state, const char *what)
 {
@@ -110,363 +122,42 @@ serial_hex_from_cert(const char *cert)
 	return ret;
 }
 
-int
-main(int argc, const char **argv)
+static int
+xml_workflow(
+	void *ctx,
+	poptContext pctx,
+	enum op_type op,
+	char *savedstate,
+	const char *eeurl, const char *agenturl,
+	dbus_bool_t use_agent_submission,
+	const char *serial, const char *template,
+	int force_new,
+	struct options *aoptions, struct options *soptions,
+	size_t num_aoptions, size_t num_soptions,
+	const char *uid,
+	const char *udn,
+	const char *pwd,
+	const char *pin,
+	dbus_bool_t use_agent_approval,
+	const char *sslcert, const char *sslkey,
+	const char *sslpin, const char *sslpinfile,
+	const char *ssldir, const char *cainfo, const char *capath,
+	dbus_bool_t can_agent,
+	int verbose
+)
 {
-	const char *eeurl = NULL, *agenturl = NULL, *url = NULL, *url2 = NULL;
 	const char *method = NULL, *method2 = NULL;
-	const char *ssldir = NULL, *cainfo = NULL, *capath = NULL;
-	const char *sslcert = NULL, *sslkey = NULL;
-	const char *sslpin = NULL, *sslpinfile = NULL;
-	const char *serial = NULL, *template = NULL;
-	const char *uid = NULL, *pwd = NULL, *pwdfile = NULL;
-	const char *udn = NULL, *pin = NULL, *pinfile = NULL;
-	char *csr = NULL, *csre = NULL;
-	char *poptarg;
-	struct {
-		char *name;
-		char *value;
-	} *aoptions = NULL, *soptions = NULL;
-	size_t num_aoptions = 0, num_soptions = 0, j;
-	char *savedstate = NULL;
+	const char *url = NULL, *url2 = NULL;
 	char *p, *q, *params = NULL, *params2 = NULL;
 	const char *lasturl = NULL, *lastparams = NULL;
-	const char *tmp = NULL, *results = NULL;
+	const char *results = NULL;
+	const char *csrfile = NULL;
 	struct cm_submit_h_context *hctx;
-	void *ctx;
-	int c, verbose = 0, force_new = 0, force_renew = 0, i;
-#ifdef DOGTAG_IPA_RENEW_AGENT
-	const char *host = NULL, *dogtag_version = NULL;
-	int eeport, agentport;
-#endif
-	enum { op_none, op_submit, op_check, op_approve, op_retrieve, op_profiles } op = op_submit;
-	dbus_bool_t can_agent, use_agent_approval = FALSE, missing_args = FALSE;
-	dbus_bool_t use_agent_submission = FALSE;
+	char *csr = NULL;
+	size_t j;
+	int i;
 	struct dogtag_default **defaults;
 	enum cm_external_status ret;
-	NSSInitContext *nctx;
-	const char *es;
-	const char *mode = CM_OP_SUBMIT, *csrfile;
-	poptContext pctx;
-	const struct poptOption popts[] = {
-		{"ee-url", 'E', POPT_ARG_STRING, &eeurl, 0, "end-entity services location", "URL"},
-		{"agent-url", 'A', POPT_ARG_STRING, &agenturl, 0, "agent services location", "URL"},
-		{"cafile", 'i', POPT_ARG_STRING, &cainfo, 0, NULL, "FILENAME"},
-		{"capath", 'C', POPT_ARG_STRING, &capath, 0, NULL, "DIRECTORY"},
-		{"dbdir", 'd', POPT_ARG_STRING, &ssldir, 0, "database containing agent or client creds", "DIRECTORY"},
-		{"nickname", 'n', POPT_ARG_STRING, &sslcert, 0, "nickname of agent or client creds", "NAME"},
-		{"certfile", 'c', POPT_ARG_STRING, &sslcert, 0, "agent or client certificate", "FILENAME"},
-		{"keyfile", 'k', POPT_ARG_STRING, &sslkey, 0, "agent or client key", "FILENAME"},
-		{"sslpinfile", 'p', POPT_ARG_STRING, &sslpinfile, 0, "agent or client key pinfile", "FILENAME"},
-		{"sslpin", 'P', POPT_ARG_STRING, &sslpin, 0, "agent or client key pin", NULL},
-		{"hex-serial", 's', POPT_ARG_STRING, NULL, 's', "request renewal for certificate by serial number (hexadecimal)", "NUMBER"},
-		{"serial", 'D', POPT_ARG_STRING, &serial, 'D', "request renewal for certificate by serial number", "NUMBER"},
-		{"submit-option", 'o', POPT_ARG_STRING, NULL, 'o', "key-value pair to send to server", NULL},
-		{"approval-option", 'O', POPT_ARG_STRING, NULL, 'O', "key-value pair to set in certificate", NULL},
-		{"profile", 'T', POPT_ARG_STRING, &template, 0, "enrollment profile", "NAME"},
-		{"profile-list", 't', POPT_ARG_NONE, NULL, 't', "list enrollment profiles", NULL},
-		{"state", 'S', POPT_ARG_STRING, &savedstate, 0, "previously-provided state data", "STATE-VALUE"},
-#ifdef DOGTAG_IPA_RENEW_AGENT
-		{"dogtag-version", 'V', POPT_ARG_STRING, &dogtag_version, 'V', NULL, "NUMBER"},
-#endif
-		{"force-new", 'N', POPT_ARG_NONE, NULL, 'N', "prefer to obtain a new certificate", NULL},
-		{"force-renew", 'R', POPT_ARG_NONE, NULL, 'R', "prefer to renew a certificate", NULL},
-		{"agent-submit", 'a', POPT_ARG_NONE, NULL, 'a', "submit enrollment or renewal request using agent or client creds", NULL},
-		{"uid", 'u', POPT_ARG_STRING, &uid, 0, "submit enrollment or renewal request using user name", "USERNAME"},
-		{"udn", 'U', POPT_ARG_STRING, &udn, 0, "submit enrollment or renewal request using user DN", "USERDN"},
-		{"userpwd", 'W', POPT_ARG_STRING, &pwd, 0, "submit password with enrollment or renewal request", NULL},
-		{"userpwdfile", 'w', POPT_ARG_STRING, &pwdfile, 0, "submit password from file with enrollment or renewal request", "FILENAME"},
-		{"userpin", 'Y', POPT_ARG_STRING, &pin, 0, "submit pin with enrollment or renewal request", NULL},
-		{"userpinfile", 'y', POPT_ARG_STRING, &pinfile, 0, "submit pin from file with enrollment or renewal request", "FILENAME"},
-		{"verbose", 'v', POPT_ARG_NONE, NULL, 'v', NULL, NULL},
-		POPT_AUTOHELP
-		POPT_TABLEEND
-	};
-
-	if (getenv(CM_SUBMIT_OPERATION_ENV) != NULL) {
-		mode = getenv(CM_SUBMIT_OPERATION_ENV);
-	}
-	if ((strcasecmp(mode, CM_OP_SUBMIT) == 0) ||
-	    (strcasecmp(mode, CM_OP_POLL) == 0)) {
-		/* fall through */
-	} else
-	if (strcasecmp(mode, CM_OP_FETCH_PROFILES) == 0) {
-		op = op_profiles;
-	} else
-	if (strcasecmp(mode, CM_OP_IDENTIFY) == 0) {
-#ifdef DOGTAG_IPA_RENEW_AGENT
-		printf("Dogtag (IPA,renew,agent) (%s %s)\n", PACKAGE_NAME,
-		       PACKAGE_VERSION);
-#else
-		printf("Dogtag (%s %s)\n", PACKAGE_NAME, PACKAGE_VERSION);
-#endif
-		return 0;
-	} else {
-		/* unsupported request */
-		return CM_SUBMIT_STATUS_OPERATION_NOT_SUPPORTED;
-	}
-
-#ifdef ENABLE_NLS
-	bindtextdomain(PACKAGE, MYLOCALEDIR);
-#endif
-
-	savedstate = getenv(CM_SUBMIT_COOKIE_ENV);
-
-	pctx = poptGetContext(argv[0], argc, argv, popts, 0);
-	if (pctx == NULL) {
-		return CM_SUBMIT_STATUS_UNCONFIGURED;
-	}
-	poptSetOtherOptionHelp(pctx, "[options] -E EE-URL -A AGENT-URL [csrfile]");
-	while ((c = poptGetNextOpt(pctx)) > 0) {
-		switch (c) {
-		case 's':
-			serial = util_dec_from_hex(poptGetOptArg(pctx));
-			break;
-		case 'O':
-			poptarg = poptGetOptArg(pctx);
-			if (strchr(poptarg, '=') == NULL) {
-				printf(_("Profile params (-O) must be in the form of param=value.\n"));
-				poptPrintUsage(pctx, stdout, 0);
-				free(soptions);
-				free(aoptions);
-				return CM_SUBMIT_STATUS_UNCONFIGURED;
-			}
-			aoptions = realloc(aoptions,
-					   ++num_aoptions * sizeof(*aoptions));
-			if (aoptions == NULL) {
-				printf(_("Out of memory.\n"));
-				free(soptions);
-				return CM_SUBMIT_STATUS_UNCONFIGURED;
-			}
-			p = strdup(poptarg);
-			if (p == NULL) {
-				printf(_("Out of memory.\n"));
-				free(aoptions);
-				free(soptions);
-				return CM_SUBMIT_STATUS_UNCONFIGURED;
-			}
-			i = strcspn(p, "=");
-			aoptions[num_aoptions - 1].name = p;
-			p[i] = '\0';
-			aoptions[num_aoptions - 1].value = p + i + 1;
-			break;
-		case 'o':
-			poptarg = poptGetOptArg(pctx);
-			if (strchr(poptarg, '=') == NULL) {
-				printf(_("Submit params (-o) must be in the form of param=value.\n"));
-				poptPrintUsage(pctx, stdout, 0);
-				free(soptions);
-				free(aoptions);
-				return CM_SUBMIT_STATUS_UNCONFIGURED;
-			}
-			soptions = realloc(soptions,
-					   ++num_soptions * sizeof(*soptions));
-			if (soptions == NULL) {
-				printf(_("Out of memory.\n"));
-				free(aoptions);
-				return CM_SUBMIT_STATUS_UNCONFIGURED;
-			}
-			p = strdup(poptarg);
-			if (p == NULL) {
-				printf(_("Out of memory.\n"));
-				free(soptions);
-				return CM_SUBMIT_STATUS_UNCONFIGURED;
-			}
-			i = strcspn(p, "=");
-			soptions[num_soptions - 1].name = p;
-			p[i] = '\0';
-			soptions[num_soptions - 1].value = p + i + 1;
-			break;
-		case 't':
-			op = op_profiles;
-			break;
-		case 'v':
-			verbose++;
-			break;
-#ifdef DOGTAG_IPA_RENEW_AGENT
-		case 'V':
-			dogtag_version = poptGetOptArg(pctx);
-			break;
-#endif
-		case 'N':
-			force_new++;
-			force_renew = 0;
-			break;
-		case 'R':
-			force_renew++;
-			force_new = 0;
-			break;
-		case 'a':
-			use_agent_submission = TRUE;
-			break;
-		}
-	}
-	if (c != -1) {
-		poptPrintUsage(pctx, stdout, 0);
-		free(soptions);
-		free(aoptions);
-		return CM_SUBMIT_STATUS_UNCONFIGURED;
-	}
-
-	umask(S_IRWXG | S_IRWXO);
-	if (isatty(STDERR_FILENO))
-		cm_log_set_method(cm_log_stderr);
-	else
-		cm_log_set_method(cm_log_syslog);
-	cm_log_set_level(verbose);
-
-	nctx = NSS_InitContext(CM_DEFAULT_CERT_STORAGE_LOCATION,
-			       NULL, NULL, NULL, NULL,
-			       NSS_INIT_NOCERTDB |
-			       NSS_INIT_READONLY |
-			       NSS_INIT_NOROOTINIT);
-	if (nctx == NULL) {
-		cm_log(1, "Unable to initialize NSS.\n");
-		_exit(1);
-	}
-	es = util_n_fips_hook();
-	if (es != NULL) {
-		cm_log(1, "Error putting NSS into FIPS mode: %s\n", es);
-		_exit(1);
-	}
-
-	ctx = talloc_new(NULL);
-
-#ifdef DOGTAG_IPA_RENEW_AGENT
-	cm_dogtag_ipa_hostver(&host, &dogtag_version);
-	if ((dogtag_version != NULL) && (atof(dogtag_version) >= 10)) {
-		eeport = 8080;
-		agentport = 8443;
-	} else {
-		eeport = 9180;
-		agentport = 9443;
-	}
-	if (eeurl == NULL) {
-		eeurl = cm_prefs_dogtag_ee_url();
-		if ((eeurl == NULL) && (host != NULL)) {
-			eeurl = talloc_asprintf(ctx,
-						"http://%s:%d/ca/ee/ca",
-						host, eeport);
-		}
-	}
-	if (agenturl == NULL) {
-		agenturl = cm_prefs_dogtag_agent_url();
-		if ((agenturl == NULL) && (host != NULL)) {
-			agenturl = talloc_asprintf(ctx,
-						   "https://%s:%d/ca/agent/ca",
-						   host, agentport);
-		}
-	}
-#endif
-
-	if (template == NULL) {
-		template = getenv(CM_SUBMIT_PROFILE_ENV);
-		if (template == NULL) {
-			template = cm_prefs_dogtag_profile();
-			if (template == NULL) {
-				/* Maybe we should ask the server for which
-				 * profiles it supports, but for now we just
-				 * assume that this one hasn't been removed. */
-				template = "caServerCert";
-			}
-		}
-	}
-	if (serial == NULL) {
-		tmp = getenv(CM_SUBMIT_CERTIFICATE_ENV);
-		if (tmp != NULL) {
-			if (cm_prefs_dogtag_renew()) {
-				serial = serial_hex_from_cert(tmp);
-				if (serial != NULL) {
-					serial = util_dec_from_hex(serial);
-				}
-			}
-		}
-	}
-	if (cainfo == NULL) {
-		cainfo = cm_prefs_dogtag_ca_info();
-	}
-	if (capath == NULL) {
-		capath = cm_prefs_dogtag_ca_path();
-	}
-	if (ssldir == NULL) {
-		ssldir = cm_prefs_dogtag_ssldir();
-	}
-	if (sslcert == NULL) {
-		sslcert = cm_prefs_dogtag_sslcert();
-	}
-	if (sslkey == NULL) {
-		sslkey = cm_prefs_dogtag_sslkey();
-	}
-	if ((sslpinfile == NULL) && (sslpin == NULL)) {
-		sslpinfile = cm_prefs_dogtag_sslpinfile();
-	}
-#ifdef DOGTAG_IPA_RENEW_AGENT
-	if ((cainfo == NULL) &&
-	    (capath == NULL) &&
-	    (ssldir == NULL) &&
-	    (sslcert == NULL) &&
-	    (sslkey == NULL) &&
-	    (sslpin == NULL) &&
-	    (sslpinfile == NULL)) {
-		cainfo = "/etc/ipa/ca.crt";
-		ssldir = "/etc/httpd/alias";
-		sslcert = "ipaCert";
-		sslpinfile = "/etc/httpd/alias/pwdfile.txt";
-	}
-#endif
-	if ((sslcert != NULL) && (strlen(sslcert) > 0)) {
-		can_agent = TRUE;
-	} else {
-		can_agent = FALSE;
-	}
-	if (use_agent_approval && !can_agent) {
-		printf(_("No agent credentials specified, and no "
-			 "default known.\n"));
-		missing_args = TRUE;
-	}
-	if (use_agent_submission && !can_agent) {
-		printf(_("No agent credentials specified, and no "
-			 "default known.\n"));
-		missing_args = TRUE;
-	}
-	if (force_renew && (serial == NULL)) {
-		printf(_("Requested renewal, but no serial number provided.\n"));
-		missing_args = TRUE;
-	}
-	if (eeurl == NULL) {
-		printf(_("No end-entity URL (-E) given, and no default known.\n"));
-		missing_args = TRUE;
-	}
-#ifdef DOGTAG_IPA_RENEW_AGENT
-	if (agenturl == NULL) {
-		printf(_("No agent URL (-A) given, and no default known.\n"));
-		missing_args = TRUE;
-	}
-#endif
-	if (template == NULL) {
-		printf(_("No profile/template (-T) given, and no default known.\n"));
-		missing_args = TRUE;
-	}
-	if (aoptions != NULL) {
-		if (agenturl == NULL) {
-			printf(_("No agent URL (-A) given, and no default "
-				 "known.\n"));
-			missing_args = TRUE;
-		}
-		if (!can_agent) {
-			printf(_("No agent credentials specified, and no "
-				 "default known.\n"));
-			missing_args = TRUE;
-		}
-	}
-	if (missing_args) {
-		poptPrintUsage(pctx, stdout, 0);
-		return CM_SUBMIT_STATUS_UNCONFIGURED;
-	}
-	if (NSS_ShutdownContext(nctx) != SECSuccess) {
-		printf(_("Error shutting down NSS.\n"));
-		return CM_SUBMIT_STATUS_UNREACHABLE;
-	}
 
 	/* Figure out where we are in the multi-step process. */
 	if ((savedstate != NULL) &&
@@ -490,21 +181,6 @@ main(int argc, const char **argv)
 		params = "";
 	}
 
-	/* Read the client password and/or PIN, if we need to. */
-	if ((pwdfile != NULL) && (pwd == NULL)) {
-		pwd = cm_submit_u_from_file(pwdfile);
-        if (pwd != NULL) {
-            pwd = talloc_strndup(ctx, pwd,
-                        strcspn(pwd, "\r\n"));
-		}
-	}
-	if ((pinfile != NULL) && (pin == NULL)) {
-		pin = cm_submit_u_from_file(pinfile);
-        if (pin != NULL) {
-            pin = talloc_strndup(ctx, pin,
-                        strcspn(pin, "\r\n"));
-		}
-	}
 
 	/* Figure out which form and arguments to use. */
 	switch (op) {
@@ -713,7 +389,6 @@ main(int argc, const char **argv)
 		}
 		results = cm_submit_h_results(hctx, NULL);
 		if (verbose > 0) {
-			fprintf(stderr, "results = \"%s\"\n", results);
 			syslog(LOG_DEBUG, "%s", results);
 		}
 		if (cm_submit_h_result_code(hctx) != 0) {
@@ -800,95 +475,855 @@ main(int argc, const char **argv)
 			       cm_submit_h_result_code(hctx),
 			       lasturl);
 		}
-		talloc_free(ctx);
 		return CM_SUBMIT_STATUS_UNREACHABLE;
 	}
 	if (results == NULL) {
 		printf(_("Internal error: no response to \"%s?%s\".\n"),
 		       lasturl, lastparams);
-		talloc_free(ctx);
 		return CM_SUBMIT_STATUS_REJECTED;
 	}
 	switch (op) {
 	case op_none:
 		printf(_("Internal error: unknown state.\n"));
-		talloc_free(ctx);
 		return CM_SUBMIT_STATUS_UNCONFIGURED;
 		break;
 	case op_submit:
 		ret = cm_submit_d_submit_eval(ctx, results, lasturl,
-					      can_agent, &p, &q);
+					      can_agent, &p, &q,
+					      CM_RPC_PROTOCOL_XML);
 		if (p != NULL) {
 			fprintf(stdout, "%s", p);
 		}
 		if (q != NULL) {
 			fprintf(stderr, "%s", q);
 		}
-		talloc_free(ctx);
 		return ret;
 		break;
 	case op_check:
 		ret = cm_submit_d_check_eval(ctx, results, lasturl,
-					     can_agent, &p, &q);
+					     can_agent, &p, &q,
+					     CM_RPC_PROTOCOL_JSON);
 		if (p != NULL) {
 			fprintf(stdout, "%s", p);
 		}
 		if (q != NULL) {
 			fprintf(stderr, "%s", q);
 		}
-		talloc_free(ctx);
 		return ret;
 		break;
 	case op_approve:
 		if (url2 == NULL) {
 			ret = cm_submit_d_approve_eval(ctx, results, lasturl,
-						       can_agent, &p, &q);
+						       &p, &q,
+						       CM_RPC_PROTOCOL_XML);
 			if (p != NULL) {
 				fprintf(stdout, "%s", p);
 			}
 			if (q != NULL) {
 				fprintf(stderr, "%s", q);
 			}
-			talloc_free(ctx);
 			return ret;
 		} else {
 			ret = cm_submit_d_review_eval(ctx, results, lasturl,
-						      can_agent, &p, &q);
+						      &p, &q);
 			if (p != NULL) {
 				fprintf(stdout, "%s", p);
 			}
 			if (q != NULL) {
 				fprintf(stderr, "%s", q);
 			}
-			talloc_free(ctx);
 			return ret;
 		}
 		break;
 	case op_retrieve:
 		ret = cm_submit_d_fetch_eval(ctx, results, lasturl,
-					     can_agent, &p, &q);
+					     &p, &q,
+					     CM_RPC_PROTOCOL_XML);
 		if (p != NULL) {
 			fprintf(stdout, "%s", p);
 		}
 		if (q != NULL) {
 			fprintf(stderr, "%s", q);
 		}
-		talloc_free(ctx);
 		return ret;
 		break;
 	case op_profiles:
-		ret = cm_submit_d_profiles_eval(ctx, results, lasturl,
-						can_agent, &p, &q);
+		ret = cm_submit_d_profiles_eval(ctx, results,
+						&p, &q,
+						CM_RPC_PROTOCOL_XML);
 		if (p != NULL) {
 			fprintf(stdout, "%s", p);
 		}
 		if (q != NULL) {
 			fprintf(stderr, "%s", q);
 		}
-		talloc_free(ctx);
 		return ret;
 		break;
 	}
-	talloc_free(ctx);
 	return CM_SUBMIT_STATUS_UNCONFIGURED;
 }
+
+static int
+ipa_workflow(
+	void *ctx,
+	poptContext pctx,
+	enum op_type op,
+	char *savedstate,
+	const char *url,
+	const char *host,
+	const char *template,
+	 __attribute__ ((unused)) struct options *aoptions,
+	 __attribute__ ((unused)) size_t num_aoptions,
+	const char *uid,
+	const char *udn,
+	const char *pwd,
+	const char *pin,
+	const char *cainfo, const char *capath,
+	dbus_bool_t can_agent,
+	int verbose
+)
+{
+	struct cm_submit_h_context *hctx;
+	char *p, *q;
+	const char *csrfile = NULL;
+	char *csr = NULL;
+	char *request_id = NULL;
+	json_t *json_req = NULL;
+	json_error_t j_error;
+	char *json_str = NULL;
+	char *referer = NULL;
+	const char *results = NULL;
+	json_t *j_root = NULL;
+	json_t *j_result_outer = NULL;
+	json_t *j_result = NULL; 
+	int i;
+	int rval = CM_SUBMIT_STATUS_UNCONFIGURED;
+	char *error_message = NULL;
+	const char *reqprinc = NULL;
+
+	reqprinc = talloc_asprintf(ctx, "host/%s", host);
+
+	/* Figure out where we are in the multi-step process. */
+	if ((savedstate != NULL) &&
+	    ((p = statevar(savedstate, "state")) != NULL) &&
+	    ((q = statevar(savedstate, "requestId")) != NULL)) {
+		if (strcmp(p, "check") == 0) {
+			op = op_check;
+		}
+		if ((strcmp(p, "review") == 0) ||
+		    (strcmp(p, "approve") == 0)) {
+			op = op_approve;
+		}
+		if ((strcmp(p, "fetch") == 0) ||
+		    (strcmp(p, "retrieve") == 0)) {
+			op = op_retrieve;
+		}
+		request_id = talloc_strdup(ctx, q);
+		free(p);
+		free(q);
+	}
+
+	/* Figure out which form and arguments to use. */
+	switch (op) {
+	case op_none:
+		printf(_("Internal error: unknown state.\n"));
+		return CM_SUBMIT_STATUS_UNCONFIGURED;
+		break;
+	case op_submit:
+		/* FIXME: need PKI to add support for submit options */
+
+		/* Fresh enrollment.  Read the CSR from the
+		 * environment, or from the command-line, that we're
+		 * going to submit for signing. */
+		csrfile = poptGetArg(pctx);
+		if (csrfile != NULL) {
+			csr = cm_submit_u_from_file(csrfile);
+		} else {
+			csr = getenv(CM_SUBMIT_CSR_ENV);
+			if (csr != NULL) {
+				csr = strdup(csr);
+			}
+		}
+		if ((csr == NULL) || (strlen(csr) == 0)) {
+			if (csrfile != NULL) {
+				printf(_("Unable to read signing request from file \"%s\".\n"),
+				       csrfile);
+			} else {
+				printf(_("Unable to read signing request from environment variable \"%s\".\n"),
+				       CM_SUBMIT_CSR_ENV);
+			}
+			poptPrintUsage(pctx, stdout, 0);
+			free(csr);
+			return CM_SUBMIT_STATUS_UNCONFIGURED;
+		}
+		/* Change the CSR from the format we get it in to the one the
+		 * server expects.  IPA just wants base64-encoded binary data,
+		 * no whitepace. */
+		p = strstr(csr, "-----BEGIN");
+		if (p != NULL) {
+			p += strcspn(p, "\n");
+			if (*p == '\n') { 
+				p++;
+			}
+			memmove(csr, p, strlen(p) + 1);
+		}
+		p = strstr(csr, "\n-----END");
+		if (p != NULL) {
+			*p = '\0';
+		}
+		while ((p = strchr(csr, '\r')) != NULL) {
+			memmove(p, p + 1, strlen(p));
+		}
+		while ((p = strchr(csr, '\n')) != NULL) {
+			memmove(p, p + 1, strlen(p));
+		}
+
+		/* The soptions are not supported by the IPA API */
+		if ((uid != NULL) || (udn != NULL) || (pwd != NULL) || (pin != NULL)) {
+			printf(_(
+				"Specifying uid, udn, pwd and pin are not supported with -J\n")
+			);
+			return CM_SUBMIT_STATUS_UNCONFIGURED;
+		}
+
+		json_req = json_pack_ex(&j_error, 0,
+								"{s:s, s:[[s], {s:s, s:s}]}",
+								"method", "cert_request",
+								"params",
+								csr,
+								"principal", reqprinc,
+								"profile_id", template);
+		break;
+	case op_check:
+		/* Check if the certificate has been issued or rejected. */
+		json_req = json_pack_ex(&j_error, 0,
+								"{s:s, s:[[s],{}]}",
+								"method", "cert_status",
+								"params",
+								request_id);
+		break;
+	case op_approve:
+		/* FIXME: need PKI to add support for approval options
+		 * Then we need to grab the original profile and do replacements.
+		 * This probably needs to happen within IPA which means we'd need
+		 * a mechanism to pass the options in.
+		 *
+		 * HINT: See cm_submit_d_xml_defaults. This might have to
+		 */
+		json_req = json_pack_ex(&j_error, 0,
+								"{s:s, s:[[s],{}]}",
+								"method", "cert_approve",
+								"params",
+								request_id);
+		break;
+	case op_retrieve:
+		/* Retrieving the new certificate. */
+		json_req = json_pack_ex(&j_error, 0,
+								"{s:s, s:[[s],{}]}",
+								"method", "cert_show",
+								"params",
+								request_id);
+		break;
+	case op_profiles:
+		json_req = json_pack_ex(&j_error, 0,
+								"{s:s, s:[[],{s:s}]}",
+								"method", "certprofile_find",
+								"params",
+								"all", "True");
+		break;
+	}
+	free(csr);
+
+	referer = talloc_asprintf(ctx, "%s", url);
+
+	/* Generate the request */
+	if (!json_req) {
+		cm_log(0, "%d json_pack_ex() failed: %s\n", op, j_error.text);
+		fprintf(stderr, "%d json_pack_ex() failed: %s\n", op, j_error.text);
+		return CM_SUBMIT_STATUS_UNCONFIGURED;
+	}
+	json_str = json_dumps(json_req, 0);
+
+	if (verbose) {
+		fprintf(stderr, "Submitting request %s\n", json_str);
+	}
+	/* Submit the request */
+	hctx = cm_submit_h_init(ctx, "POST", url, json_str,
+			"application/json", "application/json", referer,
+			cainfo, capath,
+			NULL, NULL, NULL,
+			cm_submit_h_negotiate_on,
+			cm_submit_h_delegate_off,
+			cm_submit_h_clientauth_off,
+			cm_submit_h_env_modify_off,
+			verbose > 1 ?
+			cm_submit_h_curl_verbose_on :
+			cm_submit_h_curl_verbose_off);
+
+	if (hctx == NULL) {
+		fprintf(stderr, "Error setting up JSON-RPC to %s on "
+		"the client.\n", url);
+		printf(_("Error setting up for JSON-RPC on the client.\n"));
+		rval = CM_SUBMIT_STATUS_UNCONFIGURED;
+		goto cleanup;
+	}
+
+	cm_submit_h_run(hctx);
+	if (verbose > 0) {
+		fprintf(stderr, "code = %d\n", cm_submit_h_result_code(hctx));
+		fprintf(stderr, "code_text = \"%s\"\n", cm_submit_h_result_code_text(hctx));    
+		syslog(LOG_DEBUG, "%s\n", json_str);
+	}
+
+	free(json_str);
+	if (json_req)
+		json_decref(json_req);
+
+	results = cm_submit_h_results(hctx, NULL);
+	if (verbose > 0) {
+		syslog(LOG_DEBUG, "%s", results);
+	}
+	if (cm_submit_h_response_code(hctx) != 200) {
+		cm_log(0, "JSON-RPC call failed with HTTP status code: %d\n",
+				  cm_submit_h_response_code(hctx));
+		cm_log(0, "code = %d, code_text = \"%s\"\n",
+			cm_submit_h_result_code(hctx), cm_submit_h_result_code_text(hctx));
+		rval = CM_SUBMIT_STATUS_UNREACHABLE;
+		goto cleanup;
+	}
+    i = parse_json_result(results, &error_message);
+	if (i < 0) {
+		rval = CM_SUBMIT_STATUS_UNREACHABLE;
+		goto cleanup;
+	}
+	if (i > 0) {
+		/* Interpret the error. See IPA errors.py to get the
+		 * classifications */
+		switch (i / 1000) {
+		case 2: /* authorization error - permanent */
+		case 3: /* invocation error - permanent */
+			printf("Server at %s denied our request, "
+				   "giving up: %d (%s).\n", url, i,
+				   error_message);
+			rval = CM_SUBMIT_STATUS_REJECTED;
+			goto cleanup;
+			break;
+		case 1: /* authentication error - transient? */
+		case 4: /* execution error - transient? */
+		case 5: /* generic error - transient? */
+		default:
+			printf("Server at %s failed request, "
+				   "will retry: %d (%s).\n", url, i,
+				   error_message);
+			rval = CM_SUBMIT_STATUS_UNREACHABLE;
+			goto cleanup;
+			break;
+		}
+	}
+
+	j_root = json_loads(results, 0, &j_error);
+	if (!j_root) {
+		cm_log(0, "Parsing JSON-RPC response failed: %s\n", j_error.text);
+		rval = CM_SUBMIT_STATUS_UNREACHABLE;
+		goto cleanup;
+	}
+	j_result_outer = json_object_get(j_root, "result");
+	if (!j_result_outer) {
+		cm_log(0, "Parsing JSON-RPC response failed, no outer result\n");
+		rval = CM_SUBMIT_STATUS_UNREACHABLE;
+		goto cleanup;
+	}
+
+	j_result = json_object_get(j_result_outer, "result");
+		if (!j_result) {
+	    cm_log(0, "Parsing JSON-RPC response failed, no inner result\n");
+	    rval = CM_SUBMIT_STATUS_UNREACHABLE;
+	    goto cleanup;
+	}
+
+	switch (op) {
+	case op_submit:
+		rval = cm_submit_d_submit_eval(ctx, results, NULL,
+					      can_agent, &p, &q,
+					      CM_RPC_PROTOCOL_JSON);
+		if (p != NULL) {
+			fprintf(stdout, "%s", p);
+		}
+		if (q != NULL) {
+			fprintf(stderr, "%s", q);
+		}
+		break;
+	case op_check:
+		rval = cm_submit_d_check_eval(ctx, results, NULL,
+					      can_agent, &p, &q,
+					      CM_RPC_PROTOCOL_JSON);
+		if (p != NULL) {
+			fprintf(stdout, "%s", p);
+		}
+		if (q != NULL) {
+			fprintf(stderr, "%s", q);
+		}
+		break;
+	case op_retrieve:
+		rval = cm_submit_d_fetch_eval(ctx, results, NULL,
+					     &p, &q,
+					     CM_RPC_PROTOCOL_JSON);
+		if (p != NULL) {
+			fprintf(stdout, "%s", p);
+		}
+		if (q != NULL) {
+			fprintf(stderr, "%s", q);
+		}
+		break;
+	case op_approve:
+		rval = cm_submit_d_approve_eval(ctx, results, NULL,
+						&p, &q,
+						CM_RPC_PROTOCOL_JSON);
+		if (p != NULL) {
+			fprintf(stdout, "%s", p);
+		}
+		if (q != NULL) {
+			fprintf(stderr, "%s", q);
+		}
+		break;
+	case op_profiles:
+		rval = cm_submit_d_profiles_eval(ctx, results,
+						 &p, &q,
+						 CM_RPC_PROTOCOL_JSON);
+
+		if (p != NULL) {
+			fprintf(stdout, "%s", p);
+		}
+		if (q != NULL) {
+			fprintf(stderr, "%s", q);
+		}
+		break; 
+	default:
+		break;
+	}
+
+cleanup:
+	cm_submit_h_cleanup(hctx);
+
+	return rval;
+}
+
+int
+main(int argc, const char **argv)
+{
+	const char *eeurl = NULL, *agenturl = NULL;
+	const char *jsonrpc_url = NULL;
+	const char *ssldir = NULL, *cainfo = NULL, *capath = NULL;
+	const char *sslcert = NULL, *sslkey = NULL;
+	const char *sslpin = NULL, *sslpinfile = NULL;
+	const char *serial = NULL, *template = NULL;
+	const char *uid = NULL, *pwd = NULL, *pwdfile = NULL;
+	const char *udn = NULL, *pin = NULL, *pinfile = NULL;
+	char *savedstate = NULL;
+	char *poptarg;
+	struct options *aoptions = NULL, *soptions = NULL;
+	size_t num_aoptions = 0, num_soptions = 0;
+	int c, verbose = 0, force_new = 0, force_renew = 0, i;
+	const char *host = NULL;
+#ifdef DOGTAG_IPA_RENEW_AGENT
+	const char *dogtag_version = NULL;
+	int eeport, agentport;
+#endif
+	enum op_type op = op_submit;
+	dbus_bool_t can_agent, use_agent_approval = FALSE, missing_args = FALSE;
+	dbus_bool_t use_agent_submission = FALSE;
+	NSSInitContext *nctx;
+	char *p, *tmp;
+	const char *es;
+	void *ctx;
+	const char *mode = CM_OP_SUBMIT;
+	poptContext pctx;
+	int rval;
+
+	const struct poptOption popts[] = {
+		{"ee-url", 'E', POPT_ARG_STRING, &eeurl, 0, "end-entity services location", "URL"},
+		{"agent-url", 'A', POPT_ARG_STRING, &agenturl, 0, "agent services location", "URL"},
+		{"jsonrpc-url", 'J', POPT_ARG_STRING, &jsonrpc_url, 'J', "IPA JSON-RPC service location", "URL"},
+		{"cafile", 'i', POPT_ARG_STRING, &cainfo, 0, NULL, "FILENAME"},
+		{"capath", 'C', POPT_ARG_STRING, &capath, 0, NULL, "DIRECTORY"},
+		{"dbdir", 'd', POPT_ARG_STRING, &ssldir, 0, "database containing agent or client creds", "DIRECTORY"},
+		{"nickname", 'n', POPT_ARG_STRING, &sslcert, 0, "nickname of agent or client creds", "NAME"},
+		{"certfile", 'c', POPT_ARG_STRING, &sslcert, 0, "agent or client certificate", "FILENAME"},
+		{"keyfile", 'k', POPT_ARG_STRING, &sslkey, 0, "agent or client key", "FILENAME"},
+		{"sslpinfile", 'p', POPT_ARG_STRING, &sslpinfile, 0, "agent or client key pinfile", "FILENAME"},
+		{"sslpin", 'P', POPT_ARG_STRING, &sslpin, 0, "agent or client key pin", NULL},
+		{"hex-serial", 's', POPT_ARG_STRING, NULL, 's', "request renewal for certificate by serial number (hexadecimal)", "NUMBER"},
+		{"serial", 'D', POPT_ARG_STRING, &serial, 'D', "request renewal for certificate by serial number", "NUMBER"},
+		{"submit-option", 'o', POPT_ARG_STRING, NULL, 'o', "key-value pair to send to server", NULL},
+		{"approval-option", 'O', POPT_ARG_STRING, NULL, 'O', "key-value pair to set in certificate", NULL},
+		{"profile", 'T', POPT_ARG_STRING, &template, 0, "enrollment profile", "NAME"},
+		{"profile-list", 't', POPT_ARG_NONE, NULL, 't', "list enrollment profiles", NULL},
+		{"state", 'S', POPT_ARG_STRING, &savedstate, 0, "previously-provided state data", "STATE-VALUE"},
+#ifdef DOGTAG_IPA_RENEW_AGENT
+		{"dogtag-version", 'V', POPT_ARG_STRING, &dogtag_version, 'V', NULL, "NUMBER"},
+#endif
+		{"force-new", 'N', POPT_ARG_NONE, NULL, 'N', "prefer to obtain a new certificate", NULL},
+		{"force-renew", 'R', POPT_ARG_NONE, NULL, 'R', "prefer to renew a certificate", NULL},
+		{"agent-submit", 'a', POPT_ARG_NONE, NULL, 'a', "submit enrollment or renewal request using agent or client creds", NULL},
+		{"uid", 'u', POPT_ARG_STRING, &uid, 0, "submit enrollment or renewal request using user name", "USERNAME"},
+		{"udn", 'U', POPT_ARG_STRING, &udn, 0, "submit enrollment or renewal request using user DN", "USERDN"},
+		{"userpwd", 'W', POPT_ARG_STRING, &pwd, 0, "submit password with enrollment or renewal request", NULL},
+		{"userpwdfile", 'w', POPT_ARG_STRING, &pwdfile, 0, "submit password from file with enrollment or renewal request", "FILENAME"},
+		{"userpin", 'Y', POPT_ARG_STRING, &pin, 0, "submit pin with enrollment or renewal request", NULL},
+		{"userpinfile", 'y', POPT_ARG_STRING, &pinfile, 0, "submit pin from file with enrollment or renewal request", "FILENAME"},
+		{"verbose", 'v', POPT_ARG_NONE, NULL, 'v', NULL, NULL},
+		POPT_AUTOHELP
+		POPT_TABLEEND
+	};
+
+	if (getenv(CM_SUBMIT_OPERATION_ENV) != NULL) {
+		mode = getenv(CM_SUBMIT_OPERATION_ENV);
+	}
+	if ((strcasecmp(mode, CM_OP_SUBMIT) == 0) ||
+	    (strcasecmp(mode, CM_OP_POLL) == 0)) {
+		/* fall through */
+	} else
+	if (strcasecmp(mode, CM_OP_FETCH_PROFILES) == 0) {
+		op = op_profiles;
+	} else
+	if (strcasecmp(mode, CM_OP_IDENTIFY) == 0) {
+#ifdef DOGTAG_IPA_RENEW_AGENT
+		printf("Dogtag (IPA,renew,agent) (%s %s)\n", PACKAGE_NAME,
+		       PACKAGE_VERSION);
+#else
+		printf("Dogtag (%s %s)\n", PACKAGE_NAME, PACKAGE_VERSION);
+#endif
+		return 0;
+	} else {
+		/* unsupported request */
+		return CM_SUBMIT_STATUS_OPERATION_NOT_SUPPORTED;
+	}
+
+#ifdef ENABLE_NLS
+	bindtextdomain(PACKAGE, MYLOCALEDIR);
+#endif
+
+	savedstate = getenv(CM_SUBMIT_COOKIE_ENV);
+
+	pctx = poptGetContext(argv[0], argc, argv, popts, 0);
+	if (pctx == NULL) {
+		return CM_SUBMIT_STATUS_UNCONFIGURED;
+	}
+	poptSetOtherOptionHelp(pctx, "[options] -E EE-URL -A AGENT-URL [csrfile]");
+	while ((c = poptGetNextOpt(pctx)) > 0) {
+		switch (c) {
+		case 's':
+			serial = util_dec_from_hex(poptGetOptArg(pctx));
+			break;
+		case 'O':
+			poptarg = poptGetOptArg(pctx);
+			if (strchr(poptarg, '=') == NULL) {
+				printf(_("Profile params (-O) must be in the form of param=value.\n"));
+				poptPrintUsage(pctx, stdout, 0);
+				free(soptions);
+				free(aoptions);
+				return CM_SUBMIT_STATUS_UNCONFIGURED;
+			}
+			aoptions = realloc(aoptions,
+					   ++num_aoptions * sizeof(*aoptions));
+			if (aoptions == NULL) {
+				printf(_("Out of memory.\n"));
+				free(soptions);
+				return CM_SUBMIT_STATUS_UNCONFIGURED;
+			}
+			p = strdup(poptarg);
+			if (p == NULL) {
+				printf(_("Out of memory.\n"));
+				free(aoptions);
+				free(soptions);
+				return CM_SUBMIT_STATUS_UNCONFIGURED;
+			}
+			i = strcspn(p, "=");
+			aoptions[num_aoptions - 1].name = p;
+			p[i] = '\0';
+			aoptions[num_aoptions - 1].value = p + i + 1;
+			break;
+		case 'o':
+			poptarg = poptGetOptArg(pctx);
+			if (strchr(poptarg, '=') == NULL) {
+				printf(_("Submit params (-o) must be in the form of param=value.\n"));
+				poptPrintUsage(pctx, stdout, 0);
+				free(soptions);
+				free(aoptions);
+				return CM_SUBMIT_STATUS_UNCONFIGURED;
+			}
+			soptions = realloc(soptions,
+					   ++num_soptions * sizeof(*soptions));
+			if (soptions == NULL) {
+				printf(_("Out of memory.\n"));
+				free(aoptions);
+				return CM_SUBMIT_STATUS_UNCONFIGURED;
+			}
+			p = strdup(poptarg);
+			if (p == NULL) {
+				printf(_("Out of memory.\n"));
+				free(soptions);
+				return CM_SUBMIT_STATUS_UNCONFIGURED;
+			}
+			i = strcspn(p, "=");
+			soptions[num_soptions - 1].name = p;
+			p[i] = '\0';
+			soptions[num_soptions - 1].value = p + i + 1;
+			break;
+		case 't':
+			op = op_profiles;
+			break;
+		case 'v':
+			verbose++;
+			break;
+#ifdef DOGTAG_IPA_RENEW_AGENT
+		case 'V':
+			dogtag_version = poptGetOptArg(pctx);
+			break;
+#endif
+		case 'N':
+			force_new++;
+			force_renew = 0;
+			break;
+		case 'R':
+			force_renew++;
+			force_new = 0;
+			break;
+		case 'a':
+			use_agent_submission = TRUE;
+			break;
+		}
+	}
+	if (c != -1) {
+		poptPrintUsage(pctx, stdout, 0);
+		free(soptions);
+		free(aoptions);
+		return CM_SUBMIT_STATUS_UNCONFIGURED;
+	}
+
+	umask(S_IRWXG | S_IRWXO);
+	if (isatty(STDERR_FILENO))
+		cm_log_set_method(cm_log_stderr);
+	else
+		cm_log_set_method(cm_log_syslog);
+	cm_log_set_level(verbose);
+
+	nctx = NSS_InitContext(CM_DEFAULT_CERT_STORAGE_LOCATION,
+			       NULL, NULL, NULL, NULL,
+			       NSS_INIT_NOCERTDB |
+			       NSS_INIT_READONLY |
+			       NSS_INIT_NOROOTINIT);
+	if (nctx == NULL) {
+		cm_log(1, "Unable to initialize NSS.\n");
+		_exit(1);
+	}
+	es = util_n_fips_hook();
+	if (es != NULL) {
+		cm_log(1, "Error putting NSS into FIPS mode: %s\n", es);
+		_exit(1);
+	}
+
+	ctx = talloc_new(NULL);
+
+#ifdef DOGTAG_IPA_RENEW_AGENT
+	cm_dogtag_ipa_hostver(&host, &dogtag_version);
+	if (host == NULL) {
+		printf(_("Host must be set in /etc/ipa/default.conf.\n"));
+		missing_args = TRUE;
+	}
+	if (jsonrpc_url == NULL) {
+		if ((dogtag_version != NULL) && (atof(dogtag_version) >= 10)) {
+			eeport = 8080;
+			agentport = 8443;
+		} else {
+			eeport = 9180;
+			agentport = 9443;
+		}
+		if ((eeurl == NULL) && (jsonrpc_url == NULL)) {
+			cm_log(0, "jsonrpc_url is NULL\n");
+			eeurl = cm_prefs_dogtag_ee_url();
+			if ((eeurl == NULL) && (host != NULL)) {
+				eeurl = talloc_asprintf(ctx,
+							"http://%s:%d/ca/ee/ca",
+							host, eeport);
+			}
+		}
+		if ((agenturl == NULL) && (jsonrpc_url == NULL)) {
+			cm_log(0, "jsonrpc_url is NULL\n");
+			agenturl = cm_prefs_dogtag_agent_url();
+			if ((agenturl == NULL) && (host != NULL)) {
+				agenturl = talloc_asprintf(ctx,
+							   "https://%s:%d/ca/agent/ca",
+							   host, agentport);
+			}
+		}
+	}
+#else
+	char tmphostname[255];
+	int r;
+
+	r = gethostname(tmphostname, 255 - 1);
+	if (r != 0) {
+		printf(_("gethostname() failed.\n"));
+		missing_args = TRUE;
+	}
+	host = talloc_strdup(ctx, tmphostname);
+#endif
+
+	if (template == NULL) {
+		template = getenv(CM_SUBMIT_PROFILE_ENV);
+		if (template == NULL) {
+			template = cm_prefs_dogtag_profile();
+			if (template == NULL) {
+				/* Maybe we should ask the server for which
+				 * profiles it supports, but for now we just
+				 * assume that this one hasn't been removed. */
+				template = "caServerCert";
+			}
+		}
+	}
+	if (serial == NULL) {
+		tmp = getenv(CM_SUBMIT_CERTIFICATE_ENV);
+		if (tmp != NULL) {
+			if (cm_prefs_dogtag_renew()) {
+				serial = serial_hex_from_cert(tmp);
+				if (serial != NULL) {
+					serial = util_dec_from_hex(serial);
+				}
+			}
+		}
+	}
+	if (!force_new && serial && jsonrpc_url) {
+		printf(_("Renew-by-serial is not supported over JSON-RPC.\n"));
+		missing_args = TRUE;
+	}
+	if (cainfo == NULL) {
+		cainfo = cm_prefs_dogtag_ca_info();
+	}
+	if (capath == NULL) {
+		capath = cm_prefs_dogtag_ca_path();
+	}
+	if (ssldir == NULL) {
+		ssldir = cm_prefs_dogtag_ssldir();
+	}
+	if (sslcert == NULL) {
+		sslcert = cm_prefs_dogtag_sslcert();
+	}
+	if (sslkey == NULL) {
+		sslkey = cm_prefs_dogtag_sslkey();
+	}
+	if ((sslpinfile == NULL) && (sslpin == NULL)) {
+		sslpinfile = cm_prefs_dogtag_sslpinfile();
+	}
+#ifdef DOGTAG_IPA_RENEW_AGENT
+	if ((cainfo == NULL) &&
+	    (capath == NULL)) {
+		cainfo = "/etc/ipa/ca.crt";
+	}
+
+	if ((((ssldir == NULL) &&
+	   (sslcert == NULL)) ||
+	   ((sslkey == NULL) &&
+	   (sslcert == NULL)))) {
+		printf(_("NSS database and nickname or certfile and keyfile "
+			 "must be provided.\n"));
+		missing_args = TRUE;
+	}
+#endif
+	if ((sslcert != NULL) && (strlen(sslcert) > 0)) {
+		can_agent = TRUE;
+	} else {
+		can_agent = FALSE;
+	}
+	if (use_agent_approval && !can_agent) {
+		printf(_("No agent credentials specified, and no "
+			 "default known.\n"));
+		missing_args = TRUE;
+	}
+	if (use_agent_submission && !can_agent) {
+		printf(_("No agent credentials specified, and no "
+			 "default known.\n"));
+		missing_args = TRUE;
+	}
+	if (force_renew && (serial == NULL)) {
+		printf(_("Requested renewal, but no serial number provided.\n"));
+		missing_args = TRUE;
+	}
+	if ((eeurl == NULL) && (jsonrpc_url == NULL)) {
+		printf(_("No end-entity URL (-E) given, and no default known.\n"));
+		missing_args = TRUE;
+	}
+#ifdef DOGTAG_IPA_RENEW_AGENT
+	if (agenturl == NULL && jsonrpc_url == NULL) {
+		printf(_("No agent URL (-A) given, and no default known.\n"));
+		missing_args = TRUE;
+	}
+#endif
+	if (template == NULL) {
+		printf(_("No profile/template (-T) given, and no default known.\n"));
+		missing_args = TRUE;
+	}
+	if ((aoptions != NULL) && (jsonrpc_url == NULL)) {
+		if (agenturl == NULL) {
+			printf(_("No agent URL (-A) given, and no default "
+				 "known.\n"));
+			missing_args = TRUE;
+		}
+		if (!can_agent) {
+			printf(_("No agent credentials specified, and no "
+				 "default known.\n"));
+			missing_args = TRUE;
+		}
+	}
+	if (missing_args) {
+		poptPrintUsage(pctx, stdout, 0);
+		return CM_SUBMIT_STATUS_UNCONFIGURED;
+	}
+	if (NSS_ShutdownContext(nctx) != SECSuccess) {
+		printf(_("Error shutting down NSS.\n"));
+		return CM_SUBMIT_STATUS_UNREACHABLE;
+	}
+
+	/* Read the client password and/or PIN, if we need to. */
+	if ((pwdfile != NULL) && (pwd == NULL)) {
+		pwd = cm_submit_u_from_file(pwdfile);
+        if (pwd != NULL) {
+            pwd = talloc_strndup(ctx, pwd,
+                        strcspn(pwd, "\r\n"));
+		}
+	}
+	if ((pinfile != NULL) && (pin == NULL)) {
+		pin = cm_submit_u_from_file(pinfile);
+        if (pin != NULL) {
+            pin = talloc_strndup(ctx, pin,
+                        strcspn(pin, "\r\n"));
+		}
+	}
+    if (eeurl || agenturl) {
+		rval = xml_workflow(ctx, pctx, op, savedstate, eeurl, agenturl,
+					 use_agent_submission, serial,
+					 template, force_new,
+					 aoptions, soptions, num_aoptions, num_soptions,
+					 uid, udn, pwd, pin, use_agent_approval,
+					 sslcert, sslkey, sslpin, sslpinfile, ssldir,
+					 cainfo, capath, can_agent, verbose
+		);
+	} else if (jsonrpc_url) {
+		rval = ipa_workflow(ctx, pctx, op, savedstate, jsonrpc_url,
+					 host,
+					 template,
+					 aoptions, num_aoptions,
+					 uid, udn, pwd, pin,
+					 cainfo, capath, can_agent, verbose
+		);
+	} else {
+		printf(_("None of eeurl, agenturl or jsonrpc_url are set.\n"));
+		rval = CM_SUBMIT_STATUS_UNCONFIGURED;
+	}
+
+	poptFreeContext(pctx);
+	talloc_free(ctx);
+
+	fprintf(stderr, "Returning %d\n", rval);
+	return rval;
+}
