@@ -18,6 +18,8 @@
 #ifndef cmprefs_h
 #define cmprefs_h
 
+#include <time.h>
+
 enum cm_prefs_cipher {
 	cm_prefs_aes128,
 	cm_prefs_aes192,
@@ -43,6 +45,7 @@ enum cm_key_storage_type;
 enum cm_cert_storage_type;
 
 enum cm_key_algorithm cm_prefs_preferred_key_algorithm(void);
+int cm_prefs_preferred_rsa_key_size(void);
 enum cm_prefs_cipher cm_prefs_preferred_cipher(void);
 enum cm_prefs_digest cm_prefs_preferred_digest(void);
 int cm_prefs_notify_ttls(const time_t **ttls, unsigned int *n_ttls);
@@ -72,4 +75,6 @@ const char *cm_prefs_dogtag_sslpinfile(void);
 long long prefs_key_end_of_life(time_t ref);
 long prefs_max_key_use_count(void);
 
+int cm_prefs_scep_password_otp(void);
+
 #endif
