@@ -23,6 +23,7 @@ int cm_tdbusm_get_n(DBusMessage *msg, void *parent, long *n);
 int cm_tdbusm_get_p(DBusMessage *msg, void *parent, char **p);
 int cm_tdbusm_get_s(DBusMessage *msg, void *parent, char **s);
 int cm_tdbusm_get_vs(DBusMessage *msg, void *parent, char **s);
+int cm_tdbusm_get_vn(DBusMessage *msg, void *parent, long *n);
 int cm_tdbusm_get_bp(DBusMessage *msg, void *parent, dbus_bool_t *b, char **p);
 int cm_tdbusm_get_bs(DBusMessage *msg, void *parent, dbus_bool_t *b, char **s);
 int cm_tdbusm_get_sb(DBusMessage *msg, void *parent, char **s, dbus_bool_t *b);
@@ -55,6 +56,10 @@ int cm_tdbusm_get_sssnasasasnas(DBusMessage *msg, void *parent,
 			        char **s1, char **s2, char **s3, long *n1,
 			        char ***as1, char ***as2,
 			        char ***as3, long *n2, char ***as4);
+int cm_tdbusm_get_sssnasasasnasn(DBusMessage *msg, void *parent,
+			        char **s1, char **s2, char **s3, long *n1,
+			        char ***as1, char ***as2,
+			        char ***as3, long *n2, char ***as4, long *n3);
 int cm_tdbusm_get_sasasasnas(DBusMessage *msg, void *parent,
 			     char **s,
 			     char ***as1, char ***as2,
@@ -124,6 +129,11 @@ int cm_tdbusm_set_sssnasasasnas(DBusMessage *msg,
 			        const char *s3, long n1,
 			        const char **as1, const char **as2,
 			        const char **as3, long n2, const char **as4);
+int cm_tdbusm_set_sssnasasasnasn(DBusMessage *msg,
+			        const char *s1, const char *s2,
+			        const char *s3, long n1,
+			        const char **as1, const char **as2,
+			        const char **as3, long n2, const char **as4, long n3);
 int cm_tdbusm_set_sasasasnas(DBusMessage *msg,
 			     const char *s,
 			     const char **as1, const char **as2,
