@@ -5,13 +5,13 @@ cd "$tmpdir"
 source "$srcdir"/functions
 initnssdb "$tmpdir"
 
-for size in 1024 1536 2048 3072 4096 ; do
+for size in 2048 3072 4096 ; do
 	# Generate a self-signed cert.
 	run_certutil -d "$tmpdir" -S -g $size -n keyi$size \
 		-s "cn=T$size" -c "cn=T$size" \
 		-x -t u
 	# Export the key.
-	pk12util -d "$tmpdir" -o $size.p12 -W "" -n "keyi$size" > /dev/null 2>&1
+	pk12util -C AES-128-CBC -c AES-128-CBC -d "$tmpdir" -o $size.p12 -W "" -n "keyi$size" > /dev/null 2>&1
 	openssl pkcs12 -in $size.p12 -out key.$size -passin pass: -nodes -nocerts > /dev/null 2>&1
 	cat > entry.openssl.$size <<- EOF
 	key_storage_type=FILE
@@ -30,7 +30,7 @@ for size in 1024 1536 2048 3072 4096 ; do
 	$toolsdir/keyiread entry.nss.$size
 done
 
-for size in 1024 1536 2048 3072 4096 ; do
+for size in 2048 3072 4096 ; do
 	# Generate a key.
 	openssl genrsa $size > sample.$size 2> /dev/null
 	# Check the size of the key.
